@@ -10,7 +10,7 @@ from Cookie import SimpleCookie
 
 from util import cached_property, make_traceback, obj_from_str, list_obj_from_str
 from config import cfg
-from exceptions import NotFound, BasicAuth
+from exceptions import HttpError, BasicAuth
 from routing import auto_routing, add_apps, url4
 from multidict import MultiDict, HeaderDict
 
@@ -341,12 +341,12 @@ class App(object):
             res = e
         except BasicAuth, e:
             res = e.res
-        except NotFound, e:
+        except HttpError, e:
             if cfg.DEBUG:
                 res = Response(make_traceback(rq.host),
                     code=404, content_type='text/plain')
             else:
-                res = cfg.ERROR_PAGES[404](rq)
+                res = get_error_page(rq, e[0])
         except Exception:
             tb = make_traceback(environ.get('HTTP_X_FORWARDED_HOST',
                 environ.get('HTTP_HOST', environ.get('SERVER_NAME'))))
@@ -354,7 +354,7 @@ class App(object):
             if cfg.DEBUG:
                 res = Response(tb, code=500, content_type='text/plain')
             else:
-                res = cfg.ERROR_PAGES[500](rq)
+                res = get_error_page(rq, 500)
         status, headers, body = res.wsgi()
         start_response(status, headers)
         return body
@@ -370,6 +370,9 @@ def redirect(url, permanent=False, **kwargs):
         url = url4(url, **kwargs) or url
     return Response('', code=code, headers={'Location': url})
 
+def abort(code):
+    raise HttpError(code)
+    
 
 HTTP_CODES = {
     100:    'Continue',
@@ -425,8 +428,18 @@ HTTP_CODES = {
     510:    'Not Extended'
 }
 
-STATUS_HTML = '''<html>\n<head><title>%(code)s %(status)s</title></head>
+DEFAULT_ERROR_PAGE = u'''<html>\n<head><title>%(code)s %(status)s</title></head>
 <body bgcolor="white">\n<center><h1>%(code)s %(status)s</h1></center>
 <hr><center>nginx/0.6.32</center>\n</body>\n</html>'''
-HTML_404 = STATUS_HTML % {'code': 404, 'status': HTTP_CODES[404]}
-HTML_500 = STATUS_HTML % {'code': 500, 'status': HTTP_CODES[500]}
+
+def get_error_page(rq, code):
+    try:
+        ret = cfg.ERROR_PAGES[code](rq)
+    except KeyError:
+        ret = Response(DEFAULT_ERROR_PAGE % {
+            'code': code, 'status': HTTP_CODES[code]})
+    if isinstance(ret, basestring):
+        ret = Response(ret)
+    ret.code = code
+    return ret
+
